package main

import (
	"errors"
	"flag"
	"fmt"
	"os"
	"path"
	"strings"

	"github.com/henrydcase/th5"
	hs "github.com/henrydcase/tlshandshake"
)

var tls_min, tls_max, named_groups, named_ciphers string
var dst_port string

// Parse flags
func init() {
	flag.StringVar(&tls_min, "tls_min", "1.0", "TLS version to use - minimal")
	flag.StringVar(&tls_max, "tls_max", "1.3", "TLS version to use - maximal")
	flag.StringVar(&named_groups, "groups", "", "ECDH group name to use for KEX")
	flag.StringVar(&named_ciphers, "ciphers", "", "Named cipher IDs to use")
	flag.StringVar(&dst_port, "port", "443", "Destination port number")
}

func getIDByName(m map[uint16]string, name string) (uint16, error) {
	for key, value := range m {
		if value == name {
			return key, nil
		}
	}
	return 0, errors.New("Unknown value")
}

func usage() {
	fmt.Fprintf(os.Stderr, "usage: %s [options] server:port\n", path.Base(os.Args[0]))
	fmt.Fprintf(os.Stderr, "Flags:\n")
	flag.PrintDefaults()
	os.Exit(2)
}

// Usage client args host:port
func main() {

	// parse flags
	flag.Parse()
	if flag.NArg() != 1 {
		usage()
	}

	client := hs.NewClient()
	client.Addr = flag.Arg(0)
	if !strings.Contains(client.Addr, ":") {
		client.Addr += ":" + dst_port
	}

	// Set requested DH groups
	client.TLS.CurvePreferences = []th5.CurveID{}
	if len(named_groups) != 0 {
		for _, ng := range strings.Split(named_groups, ":") {
			id, err := getIDByName(hs.NamedGroupsToName, ng)
			if err != nil {
				panic("Wrong group name provided")
			}
			client.TLS.CurvePreferences = append(client.TLS.CurvePreferences, th5.CurveID(id))
		}
	}

	// TLS min
	tlsID, err := getIDByName(hs.TlsVersionToName, tls_min)
	if err != nil {
		panic("Unknown TLS version")
	}
	client.TLS.MinVersion = tlsID

	// TLS max
	tlsID, err = getIDByName(hs.TlsVersionToName, tls_max)
	if err != nil {
		panic("Unknown TLS version")
	}
	client.TLS.MaxVersion = tlsID

	cn := strings.Split(named_ciphers, ":")
	if len(cn) == 0 {
		panic("Cipher can't be null")
	}

	if len(named_ciphers) != 0 {
		id, err := getIDByName(hs.CipherSuiteIdToName, cn[0])
		if err != nil {
			panic("Wrong cipher name provided")
		}
		client.TLS.CipherSuites = []uint16{id}
	}

	client.TLS.InsecureSkipVerify = true
	// Let's go
	client.Run()
}
